"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
// tslint:disable-next-line:ordered-imports
var path = require("path");
const checkRetrievalStatus_1 = require("../../../utils/checkRetrievalStatus");
const checkDeploymentStatus_1 = require("../../../utils/checkDeploymentStatus");
const extract_1 = require("../../../utils/extract");
const sfpowerkit_1 = require("../../../sfpowerkit");
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "package_applypatch");
class Applypatch extends command_1.SfdxCommand {
    async run() {
        this.folderPath = `temp_sfpowerkit_${fileutils_1.default.makefolderid(5)}`;
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        //Connect to the org
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        const apiversion = await conn.retrieveMaxApiVersion();
        let retrieveRequest = {
            apiVersion: apiversion,
        };
        //Retrieve Static  Resource
        retrieveRequest["singlePackage"] = true;
        retrieveRequest["unpackaged"] = {
            types: { name: "StaticResource", members: this.flags.name },
        };
        conn.metadata.pollTimeout = 60;
        let retrievedId;
        await conn.metadata.retrieve(retrieveRequest, function (error, result) {
            if (error) {
                return console.error(error);
            }
            retrievedId = result.id;
        });
        //Retrieve Patch
        let metadata_retrieve_result = await checkRetrievalStatus_1.checkRetrievalStatus(conn, retrievedId, !this.flags.json);
        if (!metadata_retrieve_result.zipFile)
            throw new core_1.SfdxError("Unable to find the requested Static Resource");
        var zipFileName = `${this.folderPath}/unpackaged.zip`;
        fs_extra_1.default.mkdirSync(this.folderPath);
        fs_extra_1.default.writeFileSync(zipFileName, metadata_retrieve_result.zipFile, {
            encoding: "base64",
        });
        if (fs_extra_1.default.existsSync(path.resolve(zipFileName))) {
            await extract_1.extract(`./${this.folderPath}/unpackaged.zip`, this.folderPath);
            fs_extra_1.default.unlinkSync(zipFileName);
            let resultFile = `${this.folderPath}/staticresources/${this.flags.name}.resource`;
            if (fs_extra_1.default.existsSync(path.resolve(resultFile))) {
                sfpowerkit_1.SFPowerkit.log(`Preparing Patch ${this.flags.name}`, sfpowerkit_1.LoggerLevel.INFO);
                fs_extra_1.default.copyFileSync(resultFile, `${this.folderPath}/unpackaged.zip`);
                //Deploy patch using mdapi
                conn.metadata.pollTimeout = 300;
                let deployId;
                var zipStream = fs_extra_1.default.createReadStream(zipFileName);
                await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
                    if (error) {
                        return console.error(error);
                    }
                    deployId = result;
                });
                sfpowerkit_1.SFPowerkit.log(`Deploying Patch with ID  ${deployId.id} to ${this.org.getUsername()}`, sfpowerkit_1.LoggerLevel.INFO);
                let metadata_deploy_result = await checkDeploymentStatus_1.checkDeploymentStatus(conn, deployId.id);
                if (!metadata_deploy_result.success) {
                    let componentFailures = metadata_deploy_result.details["componentFailures"];
                    throw new core_1.SfdxError(`Unable to deploy the Patch : ${JSON.stringify(componentFailures)}`);
                }
                sfpowerkit_1.SFPowerkit.log(`Patch ${this.flags.name} Deployed successfully.`, sfpowerkit_1.LoggerLevel.INFO);
                rimraf.sync(this.folderPath);
                return 1;
            }
            else {
                sfpowerkit_1.SFPowerkit.log(`Patch ${this.flags.name} not found in the org`, sfpowerkit_1.LoggerLevel.INFO);
                rimraf.sync(this.folderPath);
            }
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`Patch ${this.flags.name} not found in the org`, sfpowerkit_1.LoggerLevel.INFO);
            rimraf.sync(this.folderPath);
        }
    }
}
exports.default = Applypatch;
Applypatch.description = messages.getMessage("commandDescription");
Applypatch.examples = [
    `$ sfdx sfpowerkit:package:applypatch -n customer_picklist -u sandbox`,
];
Applypatch.flagsConfig = {
    name: command_1.flags.string({
        required: true,
        char: "n",
        description: messages.getMessage("nameFlagDescription"),
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL",
        ],
    }),
};
// Comment this out if your command does not require an org username
Applypatch.requiresUsername = true;
//# sourceMappingURL=applypatch.js.map