"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
const xml2js = require("xml2js");
const util = require("util");
const getPackageInfo_1 = require("../../../../utils/getPackageInfo");
const searchFilesInDirectory_1 = require("../../../../utils/searchFilesInDirectory");
const zipDirectory_1 = require("../../../../utils/zipDirectory");
const metadataFiles_1 = __importDefault(require("../../../../impl/metadata/metadataFiles"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const fileutils_1 = __importDefault(require("../../../../utils/fileutils"));
var path = require("path");
const glob = require("glob");
const spawn = require("child-process-promise").spawn;
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "source_picklist_generatepatch");
class Generatepatch extends command_1.SfdxCommand {
    async run() {
        //clean any existing temp sf powerkit source folder
        this.folderPath = `temp_sfpowerkit_${fileutils_1.default.makefolderid(5)}`;
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        // Getting Project config
        const project = await core_1.SfdxProject.resolve();
        const projectJson = await project.retrieveSfdxProjectJson();
        //Retrieve the package
        let packageToBeUsed;
        if (this.flags.package)
            packageToBeUsed = getPackageInfo_1.getPackageInfo(projectJson, this.flags.package);
        else {
            packageToBeUsed = getPackageInfo_1.getDefaultPackageInfo(projectJson);
        }
        this.flags.apiversion =
            this.flags.apiversion || projectJson.get("sourceApiVersion");
        //set objects directory
        let objectsDirPaths = glob.sync(packageToBeUsed.path + "/**/objects", {
            absolute: false,
        });
        let picklistFields = [];
        if (objectsDirPaths.length > 0) {
            for (let objectsDirPath of objectsDirPaths) {
                let fieldsInPath = await this.generatePatchForCustomPicklistField(objectsDirPath);
                if (fieldsInPath.length > 0) {
                    picklistFields = picklistFields.concat(fieldsInPath);
                    await this.generatePatchForRecordTypes(objectsDirPath);
                }
            }
        }
        if (picklistFields.length > 0) {
            await this.generateStaticResource(packageToBeUsed);
        }
        //clean temp sf powerkit source folder
        rimraf.sync(this.folderPath);
        return picklistFields;
    }
    async generatePatchForCustomPicklistField(objectsDirPath) {
        let result = [];
        sfpowerkit_1.SFPowerkit.log(`Scanning for picklist fields in ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.INFO);
        //search picklist
        let customFieldsWithPicklist = searchFilesInDirectory_1.searchFilesInDirectory(objectsDirPath, "<type>Picklist</type>", ".xml");
        //search MultiselectPicklist
        let customFieldsWithMultiPicklist = searchFilesInDirectory_1.searchFilesInDirectory(objectsDirPath, "<type>MultiselectPicklist</type>", ".xml");
        if (customFieldsWithMultiPicklist &&
            customFieldsWithMultiPicklist.length > 0) {
            customFieldsWithPicklist = customFieldsWithPicklist.concat(customFieldsWithMultiPicklist);
        }
        if (customFieldsWithPicklist && customFieldsWithPicklist.length > 0) {
            sfpowerkit_1.SFPowerkit.log(`Found ${customFieldsWithPicklist.length} picklist fields in ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.INFO);
            sfpowerkit_1.SFPowerkit.log(`Processing and adding the following fields to patch in ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.DEBUG);
            for (const file of customFieldsWithPicklist) {
                const parser = new xml2js.Parser({
                    explicitArray: false,
                });
                const parseString = util.promisify(parser.parseString);
                let field_metadata;
                try {
                    field_metadata = await parseString(fs_extra_1.default.readFileSync(path.resolve(file)));
                }
                catch (e) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to parse file ${file} due to ${e}`, sfpowerkit_1.LoggerLevel.FATAL);
                    return Promise.reject(e);
                }
                if (field_metadata.CustomField.valueSet &&
                    !field_metadata.CustomField.fieldManageability) {
                    result.push(file);
                    sfpowerkit_1.SFPowerkit.log(`Copied Original to Patch: ${file}`, sfpowerkit_1.LoggerLevel.INFO);
                    metadataFiles_1.default.copyFile(file, this.folderPath);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Added ${result.length} picklist fields into patch from ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.INFO);
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`No picklist fields found in ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.INFO);
        }
        sfpowerkit_1.SFPowerkit.log("--------------------------------------------------------------------------------", sfpowerkit_1.LoggerLevel.INFO);
        return result;
    }
    async generatePatchForRecordTypes(objectsDirPath) {
        sfpowerkit_1.SFPowerkit.log(`Scanning for recordtypes in ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.INFO);
        let recordTypes = searchFilesInDirectory_1.searchFilesInDirectory(objectsDirPath, '<RecordType xmlns="http://soap.sforce.com/2006/04/metadata">', ".xml");
        if (recordTypes && recordTypes.length > 0) {
            sfpowerkit_1.SFPowerkit.log(`Found ${recordTypes.length} RecordTypes in ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.INFO);
            sfpowerkit_1.SFPowerkit.log(`Processing and adding the following recordtypes to patch in ${objectsDirPath}`, sfpowerkit_1.LoggerLevel.INFO);
            for (const file of recordTypes) {
                sfpowerkit_1.SFPowerkit.log(`Copied Original to Patch: ${file}`, sfpowerkit_1.LoggerLevel.INFO);
                metadataFiles_1.default.copyFile(file, this.folderPath);
            }
        }
        sfpowerkit_1.SFPowerkit.log("--------------------------------------------------------------------------------", sfpowerkit_1.LoggerLevel.INFO);
        return true;
    }
    async generateStaticResource(packageToBeUsed) {
        // sfdx project json file running force source command
        var sfdx_project_json = `{	
      "packageDirectories": [	
        {	
          "path": "${packageToBeUsed.path}",	
          "default": true	
        }	
      ],	
      "namespace": "",	
      "sourceApiVersion": "${this.flags.apiversion}"	
    }`;
        fs_extra_1.default.outputFileSync(`${this.folderPath}/sfdx-project.json`, sfdx_project_json);
        if (fs_extra_1.default.existsSync(path.resolve(`${this.folderPath}/${packageToBeUsed.path}`))) {
            //Convert to mdapi
            const args = [];
            args.push("force:source:convert");
            args.push("-r");
            args.push(`${packageToBeUsed.path}`);
            args.push("-d");
            args.push(`mdapi`);
            await spawn("sfdx", args, {
                stdio: "ignore",
                cwd: this.folderPath,
            });
            //Generate zip file
            var zipFile = `${this.folderPath}/${packageToBeUsed.package}_picklist.zip`;
            await zipDirectory_1.zipDirectory(`${this.folderPath}/mdapi`, zipFile);
            //Create Static Resource Directory if not exist
            let dir = packageToBeUsed.path + `/main/default/staticresources/`;
            if (!fs_extra_1.default.existsSync(dir)) {
                fs_extra_1.default.mkdirpSync(dir);
            }
            fs_extra_1.default.copyFileSync(zipFile, `${dir}${packageToBeUsed.package}_picklist.zip`);
            //Store it to static resources
            var metadata = `<?xml version="1.0" encoding="UTF-8"?>	
      <StaticResource xmlns="http://soap.sforce.com/2006/04/metadata">	
          <cacheControl>Public</cacheControl>	
          <contentType>application/zip</contentType>	
      </StaticResource>`;
            let targetmetadatapath = `${dir}${packageToBeUsed.package}_picklist.resource-meta.xml`;
            sfpowerkit_1.SFPowerkit.log(`Generating static resource file : ${targetmetadatapath}`, sfpowerkit_1.LoggerLevel.INFO);
            fs_extra_1.default.outputFileSync(targetmetadatapath, metadata);
            sfpowerkit_1.SFPowerkit.log(`Patch ${packageToBeUsed.package}_picklist generated successfully.`, sfpowerkit_1.LoggerLevel.INFO);
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`No picklist fields found in package ${packageToBeUsed.package}`, sfpowerkit_1.LoggerLevel.WARN);
        }
    }
}
exports.default = Generatepatch;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Generatepatch.requiresProject = true;
Generatepatch.description = messages.getMessage("commandDescription");
Generatepatch.examples = [
    `$ sfdx sfpowerkit:source:picklist:generatepatch -p sfpowerkit_test`,
];
Generatepatch.flagsConfig = {
    package: command_1.flags.string({
        required: false,
        char: "p",
        description: messages.getMessage("packageFlagDescription"),
    }),
    loglevel: command_1.flags.enum({
        description: messages.getMessage("loglevel"),
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL",
        ],
    }),
};
//# sourceMappingURL=generatepatch.js.map