"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Command = exports.exec = exports.buildArgs = exports.build = exports.cmd = void 0;
const child_process_1 = require("child_process");
const fs = __importStar(require("fs-extra"));
/**
 * Create a new command.
 *
 * @param name of the command
 * @param options the command options
 */
function cmd(name, options = {}) {
    return new Command(name, options);
}
exports.cmd = cmd;
/**
 * Build a command as a string without executing it.
 *
 * Examples:
 *
 * build('npm', {version: true});
 *
 * build('java', '-version');
 *
 * @param name of the command
 * @param options the command options
 */
function build(name, ...options) {
    return cmd(name).build(...options);
}
exports.build = build;
/**
 * Build command args as a string array without executing it.
 *
 * Examples:
 *
 * buildArgs({version: true});
 *
 * buildArgs('-version');
 *
 * @param options the command options
 */
function buildArgs(...options) {
    return cmd("").buildArgs(...options);
}
exports.buildArgs = buildArgs;
/**
 * Execute a command.
 *
 * Examples:
 *
 * exec('npm', {version: true});
 *
 * exec('java', '-version');
 *
 * @param name of the command
 * @param options the command options
 */
async function exec(name, ...options) {
    return cmd(name).exec(...options);
}
exports.exec = exec;
/**
 * The command class.
 *
 * Includes methods build and exec.
 */
class Command {
    constructor(name, options = {}) {
        this.transform = defaultTransform;
        this._name = name;
        this._options = Object.assign({ cwd: process.cwd(), printCommand: true, quiet: false }, options);
    }
    /**
     * Build the command as a string without executing it.
     *
     * Examples:
     *
     * cmd('npm').build({version: true});
     *
     * cmd('java').build('-version');
     *
     * @param options the command options
     */
    build(...options) {
        const argsObj = transformOptions(options, this.transform);
        return `${this._name} ${argsObj.args.join(" ")}`;
    }
    /**
     * Build command args as a string array without executing it.
     *
     * Examples:
     *
     * cmd('npm').buildArgs({version: true});
     *
     * cmd('java').buildArgs('-version');
     *
     * @param options the command options
     */
    buildArgs(...options) {
        return transformOptions(options, this.transform).args;
    }
    /**
     * Execute the command.
     *
     * Examples:
     *
     * cmd('npm').exec({version: true});
     *
     * cmd('java').exec('-version');
     *
     * @param options the command options
     */
    async exec(...options) {
        const argsObj = transformOptions(options, this.transform);
        const quiet = isBoolean(argsObj.quiet)
            ? argsObj.quiet
            : this._options.quiet;
        const printCommand = isBoolean(argsObj.printCommand)
            ? argsObj.printCommand
            : this._options.printCommand;
        const cwd = argsObj.cwd ? argsObj.cwd : this._options.cwd;
        const execCommand = `${this._name} ${argsObj.args.join(" ")}`;
        if (!quiet && printCommand) {
            console.log(`$ ${execCommand}`);
        }
        return spawnProcess(this._name, argsObj.args, quiet, cwd);
    }
}
exports.Command = Command;
const isBoolean = (it) => [true, false].includes(it);
function isPlainObject(o) {
    return typeof o === "object" && o.constructor === Object;
}
async function spawnProcess(bin, args, quiet, cwd) {
    return new Promise((resolve, reject) => {
        const spawnOptions = {
            cwd
        };
        if (!quiet) {
            spawnOptions.stdio = "inherit";
        }
        const spawned = child_process_1.spawn(bin, args, spawnOptions);
        spawned.on("exit", exitCode => {
            exitCode === 0
                ? resolve()
                : reject(new Error(`Exit with code: ${exitCode}`));
        });
        spawned.on("error", (err) => {
            try {
                fs.statSync(cwd);
            }
            catch (e) {
                if (e.code === "ENOENT") {
                    reject(`The specified cwd does not exist: ${cwd}`);
                }
            }
            reject(err);
        });
    });
}
function transformOptions(options, transform) {
    const argsObj = {
        args: []
    };
    options.forEach((opt, idx) => {
        if (Array.isArray(opt)) {
            opt.forEach((it) => {
                if (isPlainObject(it)) {
                    argsObj.args.push(...defaultTransform(it));
                }
                else if (typeof it === "string") {
                    argsObj.args.push(it);
                }
            });
        }
        else if (isPlainObject(opt)) {
            if (idx === 0) {
                const transformed = transformFirstOption(opt, transform);
                argsObj.args.push(...transformed.args);
                argsObj.cwd = transformed.cwd;
                argsObj.printCommand = transformed.printCommand;
                argsObj.quiet = transformed.quiet;
            }
            else {
                argsObj.args.push(...defaultTransform(opt));
            }
        }
        else if (typeof opt === "string") {
            argsObj.args.push(opt);
        }
    });
    return argsObj;
}
function transformFirstOption(opt, transform) {
    const argsObj = {
        args: []
    };
    const filteredOption = {};
    Object.entries(opt).forEach(([flagName, flagValue]) => {
        if (flagName === "quiet") {
            argsObj.quiet = flagValue === true;
        }
        else if (flagName === "printCommand") {
            argsObj.printCommand = flagValue === true;
        }
        else if (flagName === "cwd") {
            argsObj.cwd = flagValue.toString();
        }
        else {
            filteredOption[flagName] = flagValue;
        }
    });
    argsObj.args.push(...transform(filteredOption));
    return argsObj;
}
function defaultTransform(opt) {
    const args = [];
    Object.entries(opt).forEach(([flagName, flagValue]) => {
        if (isBoolean(flagValue)) {
            if (flagValue === true) {
                if (flagName.length === 1) {
                    args.push(`-${flagName}`);
                }
                else {
                    args.push(`--${flagName}`);
                }
            }
        }
        else if (flagValue) {
            if (flagName.length === 1) {
                args.push(`-${flagName}`, `${flagValue}`);
            }
            else {
                args.push(`--${flagName}`, `${flagValue}`);
            }
        }
    });
    return args;
}
//# sourceMappingURL=buildArgs.js.map