"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
let request = require("request-promise-native");
const sfpowerkit_1 = require("../sfpowerkit");
let retry = require("async-retry");
const util_1 = require("util");
const child_process = require("child_process");
const ORDER_BY_FILTER = " ORDER BY CreatedDate ASC";
class ScratchOrgUtils {
    static async checkForNewVersionCompatible(hubOrg) {
        let conn = hubOrg.getConnection();
        let expectedValues = ["In Progress", "Available", "Allocate", "Assigned"];
        let availableValues = [];
        if (!this.isVersionCompatibilityChecked) {
            await retry(async (bail) => {
                const describeResult = await conn
                    .sobject("ScratchOrgInfo")
                    .describe();
                if (describeResult) {
                    for (const field of describeResult.fields) {
                        if (field.name === "Allocation_status__c" &&
                            field.picklistValues.length === 4) {
                            for (let picklistValue of field.picklistValues) {
                                if (picklistValue.active) {
                                    availableValues.push(picklistValue.value);
                                }
                            }
                            break;
                        }
                    }
                }
            }, { retries: 3, minTimeout: 30000 });
            this.isVersionCompatibilityChecked = true;
            //If there are values returned, its not compatible
            this.isNewVersionCompatible =
                expectedValues.filter((item) => {
                    return !availableValues.includes(item);
                }).length == 0
                    ? true
                    : false;
            if (!this.isNewVersionCompatible) {
                sfpowerkit_1.SFPowerkit.log(`Required Prerequisite values in ScratchOrgInfo.Allocation_status__c field is missing in the DevHub, expected values are : ${expectedValues}\n` +
                    `Switching back to previous version, we request you to update ScratchOrgInfo.Allocation_status__c field in the DevHub \n` +
                    `For more information Please refer https://github.com/Accenture/sfpowerkit/blob/main/src_saleforce_packages/scratchorgpool/force-app/main/default/objects/ScratchOrgInfo/fields/Allocation_status__c.field-meta.xml \n`, core_1.LoggerLevel.WARN);
            }
        }
        return this.isNewVersionCompatible;
    }
    static async getScratchOrgLimits(hubOrg, apiversion) {
        let conn = hubOrg.getConnection();
        var query_uri = `${conn.instanceUrl}/services/data/v${apiversion}/limits`;
        const limits = await request({
            method: "get",
            url: query_uri,
            headers: {
                Authorization: `Bearer ${conn.accessToken}`,
            },
            json: true,
        });
        sfpowerkit_1.SFPowerkit.log(`Limits Fetched: ${JSON.stringify(limits)}`, core_1.LoggerLevel.TRACE);
        return limits;
    }
    static async getScratchOrgRecordsAsMapByUser(hubOrg) {
        let conn = hubOrg.getConnection();
        let query = "SELECT count(id) In_Use, SignupEmail FROM ActiveScratchOrg GROUP BY SignupEmail ORDER BY count(id) DESC";
        const results = (await conn.query(query));
        sfpowerkit_1.SFPowerkit.log(`Info Fetched: ${JSON.stringify(results)}`, core_1.LoggerLevel.DEBUG);
        let scratchOrgRecordAsMapByUser = ScratchOrgUtils.arrayToObject(results.records, "SignupEmail");
        return scratchOrgRecordAsMapByUser;
    }
    static async getScratchOrgLoginURL(hubOrg, username) {
        let conn = hubOrg.getConnection();
        let query = `SELECT Id, SignupUsername, LoginUrl FROM ScratchOrgInfo WHERE SignupUsername = '${username}'`;
        sfpowerkit_1.SFPowerkit.log("QUERY:" + query, core_1.LoggerLevel.DEBUG);
        const results = (await conn.query(query));
        sfpowerkit_1.SFPowerkit.log(`Login URL Fetched: ${JSON.stringify(results)}`, core_1.LoggerLevel.DEBUG);
        return results.records[0].LoginUrl;
    }
    static async createScratchOrg(sfdx, id, adminEmail, config_file_path, expiry, hubOrg) {
        sfpowerkit_1.SFPowerkit.log("Parameters: " +
            id +
            " " +
            adminEmail +
            " " +
            config_file_path +
            " " +
            expiry +
            " ", core_1.LoggerLevel.TRACE);
        let result;
        try {
            if (adminEmail) {
                result = await sfdx.force.org.create({
                    quiet: false,
                    definitionfile: config_file_path,
                    setalias: `SO${id}`,
                    durationdays: expiry,
                    targetdevhubusername: hubOrg.getUsername(),
                    wait: 10,
                }, `adminEmail=${adminEmail}`);
            }
            else {
                result = await sfdx.force.org.create({
                    quiet: false,
                    definitionfile: config_file_path,
                    setalias: `SO${id}`,
                    durationdays: expiry,
                    targetdevhubusername: hubOrg.getUsername(),
                    wait: 10,
                });
            }
        }
        catch (error) {
            throw new error("Unable to create scratch org");
        }
        sfpowerkit_1.SFPowerkit.log(JSON.stringify(result), core_1.LoggerLevel.TRACE);
        let scratchOrg = {
            alias: `SO${id}`,
            orgId: result.orgId,
            username: result.username,
            signupEmail: adminEmail ? adminEmail : "",
        };
        //Get FrontDoor URL
        scratchOrg.loginURL = await this.getScratchOrgLoginURL(hubOrg, scratchOrg.username);
        //Generate Password
        scratchOrg.password = await this.generatePassword(scratchOrg.username, hubOrg.getUsername());
        sfpowerkit_1.SFPowerkit.log(JSON.stringify(scratchOrg), core_1.LoggerLevel.TRACE);
        return scratchOrg;
    }
    static async generatePassword(targetusername, targetdevhubusername) {
        let result;
        try {
            result = child_process.execSync(`npx sfdx force:user:password:generate -u ${targetusername} -v ${targetdevhubusername} --json`, { encoding: "utf8" });
        }
        catch (err) {
            sfpowerkit_1.SFPowerkit.log(JSON.stringify(err), core_1.LoggerLevel.ERROR);
            return "";
        }
        let passwordResult = "";
        let resultAsJSON = JSON.parse(result);
        if (resultAsJSON["status"] == 0) {
            passwordResult = resultAsJSON["result"]["password"];
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`Cannot generate password for SO : ${targetusername}`, core_1.LoggerLevel.ERROR);
        }
        return passwordResult;
    }
    static async shareScratchOrgThroughEmail(emailId, scratchOrg, hubOrg) {
        let hubOrgUserName = hubOrg.getUsername();
        let body = `${hubOrgUserName} has fetched a new scratch org from the Scratch Org Pool!\n
   All the post scratch org scripts have been succesfully completed in this org!\n
   The Login url for this org is : ${scratchOrg.loginURL}\n
   Username: ${scratchOrg.username}\n
   Password: ${scratchOrg.password}\n
   Please use sfdx force:auth:web:login -r ${scratchOrg.loginURL} -a <alias>  command to authenticate against this Scratch org</p>
   Thank you for using sfpowerkit!`;
        const options = {
            method: "post",
            body: JSON.stringify({
                inputs: [
                    {
                        emailBody: body,
                        emailAddresses: emailId,
                        emailSubject: `${hubOrgUserName} created you a new Salesforce org`,
                        senderType: "CurrentUser",
                    },
                ],
            }),
            url: "/services/data/v50.0/actions/standard/emailSimple",
        };
        await retry(async (bail) => {
            await hubOrg.getConnection().request(options);
        }, { retries: 3, minTimeout: 30000 });
        sfpowerkit_1.SFPowerkit.log(`Succesfully send email to ${emailId} for ${scratchOrg.username}`, core_1.LoggerLevel.INFO);
    }
    static async getScratchOrgRecordId(scratchOrgs, hubOrg) {
        if (scratchOrgs == undefined || scratchOrgs.length == 0)
            return;
        let hubConn = hubOrg.getConnection();
        let scratchOrgIds = scratchOrgs
            .map(function (scratchOrg) {
            scratchOrg.orgId = scratchOrg.orgId.slice(0, 15);
            return `'${scratchOrg.orgId}'`;
        })
            .join(",");
        let query = `SELECT Id, ScratchOrg FROM ScratchOrgInfo WHERE ScratchOrg IN ( ${scratchOrgIds} )`;
        sfpowerkit_1.SFPowerkit.log("QUERY:" + query, core_1.LoggerLevel.TRACE);
        return await retry(async (bail) => {
            const results = (await hubConn.query(query));
            let resultAsObject = this.arrayToObject(results.records, "ScratchOrg");
            sfpowerkit_1.SFPowerkit.log(JSON.stringify(resultAsObject), core_1.LoggerLevel.TRACE);
            scratchOrgs.forEach((scratchOrg) => {
                scratchOrg.recordId = resultAsObject[scratchOrg.orgId]["Id"];
            });
            return results;
        }, { retries: 3, minTimeout: 3000 });
    }
    static async setScratchOrgInfo(soInfo, hubOrg) {
        let hubConn = hubOrg.getConnection();
        sfpowerkit_1.SFPowerkit.log(JSON.stringify(soInfo), core_1.LoggerLevel.TRACE);
        return await retry(async (bail) => {
            try {
                let result = await hubConn.sobject("ScratchOrgInfo").update(soInfo);
                sfpowerkit_1.SFPowerkit.log("Setting Scratch Org Info:" + JSON.stringify(result), core_1.LoggerLevel.TRACE);
                return result.constructor !== Array ? result.success : true;
            }
            catch (err) {
                sfpowerkit_1.SFPowerkit.log("Failure at setting ScratchOrg Info" + err, core_1.LoggerLevel.TRACE);
                return false;
            }
        }, { retries: 3, minTimeout: 3000 });
    }
    static async getScratchOrgsByTag(tag, hubOrg, isMyPool, unAssigned) {
        let hubConn = hubOrg.getConnection();
        return await retry(async (bail) => {
            let query;
            if (!util_1.isNullOrUndefined(tag))
                query = `SELECT Pooltag__c, Id,  CreatedDate, ScratchOrg, ExpirationDate, SignupUsername, SignupEmail, Password__c, Allocation_status__c,LoginUrl FROM ScratchOrgInfo WHERE Pooltag__c = '${tag}'  AND Status = 'Active' `;
            else
                query = `SELECT Pooltag__c, Id,  CreatedDate, ScratchOrg, ExpirationDate, SignupUsername, SignupEmail, Password__c, Allocation_status__c,LoginUrl FROM ScratchOrgInfo WHERE Pooltag__c != null  AND Status = 'Active' `;
            if (isMyPool) {
                query =
                    query + ` AND createdby.username = '${hubOrg.getUsername()}' `;
            }
            if (unAssigned && this.isNewVersionCompatible) {
                // if new version compatible get Available / In progress
                query =
                    query +
                        `AND ( Allocation_status__c ='Available' OR Allocation_status__c = 'In Progress' ) `;
            }
            else if (unAssigned && !this.isNewVersionCompatible) {
                // if new version not compatible get not Assigned
                query = query + `AND Allocation_status__c !='Assigned' `;
            }
            query = query + ORDER_BY_FILTER;
            sfpowerkit_1.SFPowerkit.log("QUERY:" + query, core_1.LoggerLevel.TRACE);
            const results = (await hubConn.query(query));
            return results;
        }, { retries: 3, minTimeout: 3000 });
    }
    static async getActiveScratchOrgsByInfoId(hubOrg, scrathOrgIds) {
        let hubConn = hubOrg.getConnection();
        return await retry(async (bail) => {
            let query = `SELECT Id, SignupUsername FROM ActiveScratchOrg WHERE ScratchOrgInfoId IN (${scrathOrgIds}) `;
            sfpowerkit_1.SFPowerkit.log("QUERY:" + query, core_1.LoggerLevel.TRACE);
            const results = (await hubConn.query(query));
            return results;
        }, { retries: 3, minTimeout: 3000 });
    }
    static async getCountOfActiveScratchOrgsByTag(tag, hubOrg) {
        let hubConn = hubOrg.getConnection();
        return await retry(async (bail) => {
            let query = `SELECT Id, CreatedDate, ScratchOrg, ExpirationDate, SignupUsername, SignupEmail, Password__c, Allocation_status__c,LoginUrl FROM ScratchOrgInfo WHERE Pooltag__c = '${tag}' AND Status = 'Active' `;
            sfpowerkit_1.SFPowerkit.log("QUERY:" + query, core_1.LoggerLevel.TRACE);
            const results = (await hubConn.query(query));
            sfpowerkit_1.SFPowerkit.log("RESULT:" + JSON.stringify(results), core_1.LoggerLevel.TRACE);
            return results.totalSize;
        }, { retries: 3, minTimeout: 3000 });
    }
    static async getCountOfActiveScratchOrgsByTagAndUsername(tag, hubOrg) {
        let hubConn = hubOrg.getConnection();
        return await retry(async (bail) => {
            let query = `SELECT Id, CreatedDate, ScratchOrg, ExpirationDate, SignupUsername, SignupEmail, Password__c, Allocation_status__c,LoginUrl FROM ScratchOrgInfo WHERE Pooltag__c = '${tag}' AND Status = 'Active' `;
            sfpowerkit_1.SFPowerkit.log("QUERY:" + query, core_1.LoggerLevel.TRACE);
            const results = (await hubConn.query(query));
            return results.totalSize;
        }, { retries: 3, minTimeout: 3000 });
    }
    static async getActiveScratchOrgRecordIdGivenScratchOrg(hubOrg, apiversion, scratchOrgId) {
        let hubConn = hubOrg.getConnection();
        return await retry(async (bail) => {
            var query_uri = `${hubConn.instanceUrl}/services/data/v${apiversion}/query?q=SELECT+Id+FROM+ActiveScratchOrg+WHERE+ScratchOrg+=+'${scratchOrgId}'`;
            const result = await request({
                method: "get",
                url: query_uri,
                headers: {
                    Authorization: `Bearer ${hubConn.accessToken}`,
                },
                json: true,
            });
            sfpowerkit_1.SFPowerkit.log("Retrieve Active ScratchOrg Id:" + JSON.stringify(result), core_1.LoggerLevel.TRACE);
            return result.records[0].Id;
        }, { retries: 3, minTimeout: 3000 });
    }
    static async deleteScratchOrg(hubOrg, scratchOrgIds) {
        let hubConn = hubOrg.getConnection();
        await retry(async (bail) => {
            await hubConn.sobject("ActiveScratchOrg").del(scratchOrgIds);
        }, { retries: 3, minTimeout: 3000 });
    }
    static async checkForPreRequisite(hubOrg) {
        let hubConn = hubOrg.getConnection();
        return await retry(async (bail) => {
            const results = await hubConn.metadata.read("CustomObject", "ScratchOrgInfo");
            const checker = (element) => element.fullName === "Allocation_status__c";
            sfpowerkit_1.SFPowerkit.log(JSON.stringify(results), core_1.LoggerLevel.TRACE);
            if (results["fields"].some(checker)) {
                return true;
            }
            else {
                return false;
            }
        }, { retries: 3, minTimeout: 2000 });
    }
}
exports.default = ScratchOrgUtils;
ScratchOrgUtils.isNewVersionCompatible = false;
ScratchOrgUtils.isVersionCompatibilityChecked = false;
ScratchOrgUtils.arrayToObject = (array, keyfield) => array.reduce((obj, item) => {
    obj[item[keyfield]] = item;
    return obj;
}, {});
//# sourceMappingURL=scratchOrgUtils.js.map